@@ -1,6 +1,7 @@
 import numpy as np
 import copy
 import pickle as pickle
+import time
 
 from .results import history
 from .. import discrete
@@ -62,8 +63,24 @@ class policy(discrete.policy):
             self.mpirank = comm.rank
             self.actions = np.array_split(self.actions, self.mpisize)[self.mpirank]
 
-    def write(self, action, t, X=None):
-        self.history.write(t, action)
+    def write(
+        self,
+        action,
+        t,
+        X=None,
+        time_total=None,
+        time_update_predictor=None,
+        time_get_action=None,
+        time_run_simulator=None,
+    ):
+        self.history.write(
+            t,
+            action,
+            time_total=time_total,
+            time_update_predictor=time_update_predictor,
+            time_get_action=time_get_action,
+            time_run_simulator=time_run_simulator,
+        )
         action = np.array(action)
         t = np.array(t)
 
@@ -88,7 +105,12 @@ class policy(discrete.policy):
         predictor = self.predictor_list[i]
         test = self.test_list[i]
         new_data = self.new_data_list[i]
-        return {"training": training, "predictor": predictor, "test": test, "new_data": new_data}
+        return {
+            "training": training,
+            "predictor": predictor,
+            "test": test,
+            "new_data": new_data,
+        }
 
     def random_search(
         self,
@@ -109,18 +131,32 @@ class policy(discrete.policy):
 
         for n in range(0, max_num_probes):
 
+            time_total = time.time()
             if is_disp and N > 1:
-                utility.show_start_message_multi_search_mo(
-                    self.history.num_runs, "random"
+                utility.show_start_message_multi_search(
+                    self.history.num_runs, score="random"
                 )
 
+            time_get_action = time.time()
             action = self._get_random_action(N)
+            time_get_action = time.time() - time_get_action
 
             if simulator is None:
                 return action
 
+            time_run_simulator = time.time()
             t = _run_simulator(simulator, action, self.mpicomm)
-            self.write(action, t)
+            time_run_simulator = time.time() - time_run_simulator
+
+            time_total = time.time() - time_total
+            self.write(
+                action,
+                t,
+                time_total=[time_total] * N,
+                time_update_predictor=np.zeros(N, dtype=float),
+                time_get_action=[time_get_action] * N,
+                time_run_simulator=[time_run_simulator] * N,
+            )
 
             if is_disp:
                 utility.show_search_results_mo(
@@ -146,6 +182,9 @@ class policy(discrete.policy):
         if self.mpirank != 0:
             is_disp = False
 
+        old_disp = self.config.learning.is_disp
+        self.config.learning.is_disp = is_disp
+
         if max_num_probes is None:
             max_num_probes = 1
             simulator = None
@@ -173,35 +212,54 @@ class policy(discrete.policy):
         N = int(num_search_each_probe)
 
         for n in range(max_num_probes):
-
+            time_total = time.time()
+            time_update_predictor = time.time()
             if utility.is_learning(n, interval):
                 self._learn_hyperparameter(num_rand_basis)
             else:
                 self._update_predictor()
+            time_update_predictor = time.time() - time_update_predictor
 
             if num_search_each_probe != 1:
-                utility.show_start_message_multi_search_mo(self.history.num_runs, score)
+                utility.show_start_message_multi_search(self.history.num_runs, score)
 
+            time_get_action = time.time()
             K = self.config.search.multi_probe_num_sampling
             alpha = self.config.search.alpha
             action = self._get_actions(score, N, K, alpha)
+            time_get_action = time.time() - time_get_action
 
-            if len(action) == 0:
+            N_indeed = len(action)
+            if N_indeed == 0:
                 if self.mpirank == 0:
                     print("WARNING: All actions have already searched.")
+                self.config.learning.is_disp = old_disp
                 return copy.deepcopy(self.history)
 
             if simulator is None:
+                self.config.learning.is_disp = old_disp
                 return action
 
+            time_run_simulator = time.time()
             t = _run_simulator(simulator, action, self.mpicomm)
-            self.write(action, t)
+            time_run_simulator = time.time() - time_run_simulator
+
+            time_total = time.time() - time_total
+            self.write(
+                action,
+                t,
+                time_total=[time_total] * N_indeed,
+                time_update_predictor=[time_update_predictor] * N_indeed,
+                time_get_action=[time_get_action] * N_indeed,
+                time_run_simulator=[time_run_simulator] * N_indeed,
+            )
 
             if is_disp:
                 utility.show_search_results_mo(
                     self.history, N, disp_pareto_set=disp_pareto_set
                 )
         self._update_predictor()
+        self.config.learning.is_disp = old_disp
         return copy.deepcopy(self.history)
 
     def _get_actions(self, mode, N, K, alpha):
@@ -214,13 +272,13 @@ class policy(discrete.policy):
 
         chosen_actions = [champion]
         for n in range(1, N):
-            f = self.get_score(mode=mode, alpha=alpha, parallel=False)
+            f = self._get_marginal_score(mode, chosen_actions[0:n], K, alpha)
             champion, local_champion, local_index = self._find_champion(f)
             if champion == -1:
                 break
-            chosen_actions.append(champion)
             if champion == local_champion:
                 self.actions = self._delete_actions(local_index)
+            chosen_actions.append(champion)
         return np.array(chosen_actions)
 
     def get_post_fmean(self, xs):
@@ -311,11 +369,69 @@ class policy(discrete.policy):
             reduced_candidate_num=self.TS_candidate_num,
             alpha=alpha,
         )
-        if parallel and self.mpisize>1:
+        if parallel and self.mpisize > 1:
             fs = self.mpicomm.allgather(f)
             f = np.hstack(fs)
         return f
 
+    def _get_marginal_score(self, mode, chosen_actions, K, alpha):
+        """
+        Getting marginal scores.
+
+        Parameters
+        ----------
+        mode: str
+            The type of aquision funciton.
+            TS (Thompson Sampling), EI (Expected Improvement) and PI (Probability of Improvement) are available.
+            These functions are defined in score.py.
+        chosen_actions: numpy.ndarray
+            Array of selected actions.
+        K: int
+            The total number of search candidates.
+        alpha: float
+            not used.
+
+        Returns
+        -------
+        f: list
+            N dimensional scores (score is defined in each mode)
+        """
+        f = np.zeros((K, len(self.actions)), dtype=float)
+
+        # draw K samples of the values of objective function of chosen actions
+        new_test_list = [variable() for _ in range(self.num_objectives)]
+        virtual_t_list = [np.zeros((K, 0)) for _ in range(self.num_objectives)]
+        for i in range(self.num_objectives):
+            new_test_local = self.test_list[i].get_subset(chosen_actions)
+            virtual_t_local = self.predictor_list[i].get_predict_samples(self.training_list[i], new_test_local, K)
+            if self.mpisize == 1:
+                new_test_list[i] = new_test_local
+                virtual_t_list[i] = virtual_t_local
+            else:
+                for nt in self.mpicomm.allgather(new_test_local):
+                    new_test_list[i].add(X=nt.X, t=nt.t, Z=nt.Z)
+                virtual_t_list[i] = np.concatenate(self.mpicomm.allgather(virtual_t_local), axis=1)
+
+        for k in range(K):
+            predictor_list = [copy.deepcopy(p) for p in self.predictor_list]
+            training_list = [copy.deepcopy(t) for t in self.training_list]
+
+            for i in range(self.num_objectives):
+                virtual_train = new_test_list[i]
+                virtual_train.t = virtual_t_list[i][k, :]
+
+                if virtual_train.Z is None:
+                    training_list[i].add(virtual_train.X, virtual_train.t)
+                else:
+                    training_list[i].add(virtual_train.X, virtual_train.t, virtual_train.Z)
+
+                predictor_list[i].update(training_list[i], virtual_train)
+
+            f[k, :] = self.get_score(
+                mode, predictor_list=predictor_list, training_list=training_list, parallel=False
+            )
+        return np.mean(f, axis=0)
+
     def save(self, file_history, file_training_list=None, file_predictor_list=None):
         if self.mpirank == 0:
             self.history.save(file_history)
@@ -387,7 +503,9 @@ class policy(discrete.policy):
     def _update_predictor(self):
         for i in range(self.num_objectives):
             if self.new_data_list[i] is not None:
-                self.predictor_list[i].update(self.training_list[i], self.new_data_list[i])
+                self.predictor_list[i].update(
+                    self.training_list[i], self.new_data_list[i]
+                )
                 self.new_data_list[i] = None
 
 
